/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.extension.brick;

import org.fortiss.af3.component.model.InputPort;
import org.fortiss.af3.platform.raspberry.generator.framework.IReadableGeneratorExtension;
import org.fortiss.af3.platform.raspberry.model.brick.LaserRangeSensor;

/**
 * Generator extension for {@link LaserRangeSensor}s.
 * 
 * @author hoelzl
 */
public class LaserRangeSensorGeneratorExtension extends
		InstanceBrickGeneratorExtensionBase<LaserRangeSensor> implements
		IReadableGeneratorExtension {
	/** Constructor. */
	public LaserRangeSensorGeneratorExtension(LaserRangeSensor modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public String getHeaderFileName() {
		return "bricklet_laser_range_finder.h";
	}

	/** {@inheritDoc} */
	@Override
	public String getInstanceVariableDeclaration() {
		String postfix = getInstancePostfix();
		StringBuilder sb = new StringBuilder();
		sb.append("static LaserRangeFinder laser_range_device_" + postfix + ";\n");
		sb.append("static uint16_t laser_range_value_" + postfix + " = 0;\n");
		sb.append("static uint64_t laser_range_last_cb_time_" + postfix + " = 0;\n");
		return sb.toString();
	}

	/** {@inheritDoc} */
	@Override
	public String getInstanceAuxiliaryFunctions() {
		String postfix = getInstancePostfix();
		StringBuilder sb = new StringBuilder();
		sb.append("void laser_range_callback_" + postfix + "(uint16_t distance, void *data) {\n");
		sb.append("laser_range_value_" + postfix + " = distance;\n");
		sb.append("laser_range_last_cb_time_" + postfix + " = time_util_get_current_micros();\n");
		sb.append("}\n\n");
		return sb.toString();
	}

	/** {@inheritDoc} */
	@Override
	public String getInstanceInitializationCode() {
		String postfix = getInstancePostfix();
		StringBuilder sb = new StringBuilder();
		sb.append("// initialize " + modelElement.getName());
		sb.append("laser_range_finder_create(&laser_range_device_" + postfix + ", \"" +
				modelElement.getUniqueBrickletID() + "\", &" + getConnectionVariable() + ");\n");
		sb.append("laser_range_finder_enable_laser(&laser_range_device_" + postfix + ");\n");
		sb.append("laser_range_finder_register_callback(&laser_range_device_" + postfix +
				", LASER_RANGE_FINDER_CALLBACK_DISTANCE, (void*)laser_range_callback_" + postfix +
				", NULL);\n");
		sb.append("laser_range_finder_set_distance_callback_period(&laser_range_device_" + postfix +
				", 10);\n");
		return sb.toString();
	}

	/** {@inheritDoc} */
	@Override
	public String getReadCode(InputPort logicalSignal, String targetVariable) {
		StringBuilder sb = new StringBuilder();
		sb.append(targetVariable + " = " + "laser_range_value_" + getInstancePostfix() + ";\n");
		return sb.toString();
	}

	/** {@inheritDoc} */
	@Override
	public String getNoValReadCode(InputPort logicalSignal) {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getInstanceTerminationCode() {
		return null;
	}
}
