/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.extension.controlcenter;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.platform.raspberry.generator.extension.library.PiHALLibraryGeneratorExtensionBase;
import org.fortiss.af3.platform.raspberry.generator.framework.ISharedInstanceGeneratorExtension;
import org.fortiss.af3.platform.raspberry.model.impl.RaspberryPiImpl;

/**
 * Base class for all executables of the controlcenter .
 * 
 * @author duenhuber
 */
abstract class ControlcenterGeneratorExtensionBase<T extends EObject> extends
		PiHALLibraryGeneratorExtensionBase<T> implements ISharedInstanceGeneratorExtension {
	/** Stores the instance postfix. */
	private String sharedInstancePostfix;

	/** {@inheritDoc} */
	@Override
	public String getSharedInstanceIdentifier() {
		// TODO Auto-generated method stub
		return sharedInstancePostfix;
	}

	/** {@inheritDoc} */
	@Override
	public void setSharedInstancePostfix(String sharedInstancePostfix) {
		// TODO Auto-generated method stub
		this.sharedInstancePostfix = sharedInstancePostfix;
	}

	/** {@inheritDoc} */
	@Override
	public String getSharedInstanceVariableDeclarationCode() {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getSharedInstanceAuxiliaryFunctions() {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getSharedInstanceInitializationCode() {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getSharedInstanceTerminationCode() {
		// TODO Auto-generated method stub
		return null;
	}

	/** Constructor. */
	public ControlcenterGeneratorExtensionBase(T modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public final String getSingletonVariableDeclarationCode() {
		// no declartion needed yet
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public final String getSingletonAuxiliaryFunctions() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public final String getSingletonInitializationCode() {
		// get model
		RaspberryPiImpl piimpl = (RaspberryPiImpl)modelElement.eContainer();
		// build string
		StringBuilder sb = new StringBuilder();
		sb.append("// initialize the control center configuration\n");
		sb.append("af3_module_initialize(\"" + piimpl.getName() + "\", 0, NULL, NULL );\n");
		sb.append("af3_cc_activate(\"" + piimpl.getControlcenterServerAddress() + "\"," +
				piimpl.getControlcenterServerPort() + ");\n\n");
		return sb.toString();
	}

	/** {@inheritDoc} */
	@Override
	public final String getSingletonTerminationCode() {
		return "af3_cc_deactivate();\n";
	}

	/** {@inheritDoc} */
	@Override
	public final String getSingletonIdentifier() {
		// use header file as singleton identifier
		return getHeaderFileName();
	}

	/** {@inheritDoc} */
	@Override
	public final String getHeaderFileName() {
		return "control_center_downstream.h";
	}

	/** {@inheritDoc} */
	@Override
	public String getSharedInstanceReadCode() {
		// clean cc donwstream
		return "af3_cc_downstream_clean_up_rx_list( 3 ); \n";
	}

	/** {@inheritDoc} */
	@Override
	public String getSharedInstanceWriteCode() {
		// TODO Auto-generated method stub
		return null;
	}

}
