/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.framework;

import java.util.List;

/**
 * Interface for generator extensions, which require one or more library files to be included during
 * the linker pass. This is usually the case when the libraries in question are pre-installed
 * (e.g. C standard libraries). Note that the generator will automatically add the "lib" prefix when
 * necessary. Also the implementor is required to provide all library names including any
 * dependencies: if {@code LibA} depends on {@code LibB} then returned list must be {@code LibA}
 * followed by {@code LibB}, i.e., libraries must be included before libraries they depend on.
 * 
 * @author hoelzl
 */
public interface IPreinstalledLibraryGeneratorExtension {
	/** Returns the list of library files. */
	public List<String> getLibraryNames();
}
